package evgeniy.martian.base.app;

import evgeniy.martian.base.level.Level;
import evgeniy.martian.base.level.tile.Tile;
import evgeniy.martian.base.objects.Manager;
import evgeniy.martian.base.objects.ResourceManager;
import evgeniy.martian.base.utils.MutablePosition;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

public class LevelRenderer extends JPanel implements KeyListener {

    public static final int TILE_SIZE = 32;

    private final UiApplication app;

    private final Manager manager;
    private final ResourceManager resourceManager;

    private MutablePosition cameraOffset = new MutablePosition(0, 0);

    public LevelRenderer(UiApplication app, Manager manager, ResourceManager resourceManager) {
        this.app = app;

        this.manager = manager;
        this.resourceManager = resourceManager;
    }

    @Override
    protected void paintComponent(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());

        Level level = manager.getLevel();
        if (level == null) return;

        for (int x = 0; x < level.getHeight(); x++) {
            for (int y = 0; y < level.getWidth(); y++) {
                Tile tile = level.get(x, y);
                Image tileImage = resourceManager.getFor(tile);
                if (tileImage == null) return;

                g.drawImage(tileImage, cameraOffset.getX() + y * TILE_SIZE, cameraOffset.getY() + x * TILE_SIZE, TILE_SIZE, TILE_SIZE, null);
            }
        }

        Image character = resourceManager.getCharacter();
        if (character != null) g.drawImage(character, cameraOffset.getX() + manager.getPlayerPosition().getY() * TILE_SIZE, cameraOffset.getY() + manager.getPlayerPosition().getX() * TILE_SIZE, TILE_SIZE, TILE_SIZE, null);
    }

    public MutablePosition getCameraOffset() {
        return this.cameraOffset;
    }

    @Override
    public void keyTyped(KeyEvent e) {

    }

    @Override
    public void keyPressed(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_W:
                cameraOffset.plus(0, -TILE_SIZE);
                updateUI();
                break;
            case KeyEvent.VK_S:
                cameraOffset.plus(0, TILE_SIZE);
                updateUI();
                break;
            case KeyEvent.VK_A:
                cameraOffset.plus(-TILE_SIZE, 0);
                updateUI();
                break;
            case KeyEvent.VK_D:
                cameraOffset.plus(TILE_SIZE, 0);
                updateUI();
                break;
        }
    }

    @Override
    public void keyReleased(KeyEvent e) {

    }
}
