package evgeniy.martian.base.level;

import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.KeyTile;
import evgeniy.martian.base.level.tile.Tile;
import evgeniy.martian.base.utils.Position;

import java.util.ArrayList;
import java.util.List;

public class Level {

    private final int width;
    private final int height;

    private final Tile[][] level;

    private Position startPosition;
    private Position endPosition;

    public Level(int height, int width) {
        this.height = height;
        this.width = width;

        this.level = new Tile[height][width];
    }

    public int getWidth() {
        return this.width;
    }

    public int getHeight() {
        return this.height;
    }

    public void set(int x, int y, Tile tile) {
        level[x][y] = tile;
    }

    public Tile get(int x, int y) {
        return level[x][y];
    }

    public Tile get(Position position) {
        return get(position.getX(), position.getY());
    }

    public Position getStartPosition() {
        return this.startPosition;
    }

    public void setStartPosition(Position position) {
        this.startPosition = position;
    }

    public Position getEndPosition() {
        return this.endPosition;
    }

    public void setEndPosition(Position endPosition) {
        this.endPosition = endPosition;
    }

    public Position getKey(DoorTile.Variant variant) {
        return getKey(variant.getKey());
    }

    public Position getKey(KeyTile.Variant variant) {
        for (int x = 0; x < height; x++) {
            for (int y = 0; y < width; y++) {
                Tile tile = get(x, y);
                if (tile.is(Tile.Type.KEY) && ((KeyTile) tile).getVariant() == variant) {
                    return new Position(x, y);
                }
            }
        }
        return null;
    }

    public List<Position> getHealths() {
        List<Position> result = new ArrayList<>();
        for (int x = 0; x < height; x++) {
            for (int y = 0; y < width; y++) {
                Tile tile = get(x, y);
                if (tile.is(Tile.Type.HEALTH)) {
                    result.add(new Position(x, y));
                }
            }
        }
        return result;
    }

    public List<Position> getDoors() {
        List<Position> result = new ArrayList<>();
        for (int x = 0; x < height; x++) {
            for (int y = 0; y < width; y++) {
                Tile tile = get(x, y);
                if (tile.is(Tile.Type.DOOR)) {
                    result.add(new Position(x, y));
                }
            }
        }
        return result;
    }

    public void dump() {
        StringBuilder builder = new StringBuilder();
        for (int x = 0; x < this.level.length; x++) {
            Tile[] xTiles = this.level[x];
            for (int y = 0; y < xTiles.length; y++) {
                Tile tile = xTiles[y];
                if (tile == null) builder.append("?");
                else builder.append(tile.name());
            }
            builder.append("\n");
        }
        System.out.println(builder);
    }
}
