package evgeniy.martian.base.objects;

import evgeniy.martian.base.level.Level;
import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.Tile;
import evgeniy.martian.base.utils.GoalsFinder;
import evgeniy.martian.base.utils.Position;

import java.util.ArrayDeque;
import java.util.Deque;

import static evgeniy.martian.base.objects.pathfinding.AStarPathfinding.aStar;

public class MoveTaskManager {

    private final Manager manager;

    private Deque<Position> goals;
    private Deque<Position> currentPath;
    private Level level;

    private Deque<Position> fullPath;
    private int steps = 0;

    public MoveTaskManager(Manager manager) {
        this.manager = manager;
    }

    public Deque<Position> getFullPath() {
        return this.fullPath;
    }

    public void initializeMap() {
        level = manager.getLevel();
        fullPath = new ArrayDeque<>();

        GoalsFinder finder = new GoalsFinder(manager, level);
        goals = finder.findGoals();

        fullPath.addLast(level.getStartPosition());
    }

    public Status step() {
        if (currentPath == null || currentPath.isEmpty()) {
            if (goals == null) return Status.FAILURE;
            if (level == null) return Status.FAILURE;

            if (goals.isEmpty()) {
                if (level.getEndPosition().equals(manager.getPlayerPosition().toImmutable())) {
                    return Status.FINISHED;
                }
                return Status.PATH_NOT_FOUND;
            }

            Position playerPosition = manager.getPlayerPosition().toImmutable();
            Position goal = goals.removeFirst();

            currentPath = aStar(manager, playerPosition, goal);
            if (!currentPath.isEmpty()) currentPath.removeFirst();
        }

        Position goal = currentPath.removeFirst();
        Tile tile = level.get(goal);

        if (tile.is(Tile.Type.DOOR)) {
            DoorTile door = (DoorTile) tile;
            if (!manager.getInventory().hasKey(door.getVariant().getKey())) {
                return Status.PATH_NOT_FOUND;
            }
        }

        fullPath.addLast(goal);
        manager.getPlayerPosition().setFrom(goal);
        tile.standOn(manager);

        steps += 1;
        manager.updateSteps(steps);

        if (manager.getPlayerHealth() < 1) {
            return Status.DEAD;
        }

        return Status.SUCCESS;
    }

    public enum Status {

        FINISHED,
        FAILURE,
        DEAD,
        PATH_NOT_FOUND,
        SUCCESS

    }

}
