package evgeniy.martian.base.objects;

import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.FireTile;
import evgeniy.martian.base.level.tile.KeyTile;
import evgeniy.martian.base.level.tile.Tile;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.IOException;

public class ResourceManager {

    private Image character;

    private Image floor;
    private Image wall;
    private Image start;
    private Image exit;
    private Image health;

    private Image doorA;
    private Image doorB;
    private Image doorC;
    private Image doorD;
    private Image doorE;

    private Image keyA;
    private Image keyB;
    private Image keyC;
    private Image keyD;
    private Image keyE;

    private Image fire1;
    private Image fire2;
    private Image fire3;
    private Image fire4;
    private Image fire5;

    private final Manager manager;

    public ResourceManager(Manager manager) {
        this.manager = manager;
    }

    public void load() {
        character = load("/assets/character.png");

        floor = load("/assets/floor.png");
        wall = load("/assets/wall.png");
        start = load("/assets/floor.png");
        exit = load("/assets/exit.png");
        health = load("/assets/health.png");

        doorA = load("/assets/doorA.png");
        doorB = load("/assets/doorB.png");
        doorC = load("/assets/doorC.png");
        doorD = load("/assets/doorD.png");
        doorE = load("/assets/doorE.png");

        keyA = load("/assets/keyA.png");
        keyB = load("/assets/keyB.png");
        keyC = load("/assets/keyC.png");
        keyD = load("/assets/keyD.png");
        keyE = load("/assets/keyE.png");

        fire1 = load("/assets/fire1.png");
        fire2 = load("/assets/fire2.png");
        fire3 = load("/assets/fire3.png");
        fire4 = load("/assets/fire4.png");
        fire5 = load("/assets/fire5.png");
    }

    private Image load(String path) {
        try {
            return ImageIO.read(getClass().getResource(path));
        }
        catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Image getFor(Tile tile) {
        if (tile == null) return null;
        switch (tile.getType()) {
            case FLOOR: return floor;
            case WALL: return wall;
            case START: return start;
            case EXIT: return exit;
            case DOOR:
                DoorTile.Variant doorVariant = ((DoorTile) tile).getVariant();
                if (manager.getInventory().hasKey(doorVariant.getKey())) {
                    return floor;
                }
                switch (doorVariant) {
                    case A: return doorA;
                    case B: return doorB;
                    case C: return doorC;
                    case D: return doorD;
                    case E: return doorE;
                    default: return null;
                }
            case KEY:
                KeyTile.Variant keyVariant = ((KeyTile) tile).getVariant();
                if (manager.getInventory().hasKey(keyVariant)) {
                    return floor;
                }
                switch (keyVariant) {
                    case A: return keyA;
                    case B: return keyB;
                    case C: return keyC;
                    case D: return keyD;
                    case E: return keyE;
                    default: return null;
                }
            case FIRE:
                int damageMultiplier = ((FireTile) tile).getDamageMultiplier();
                switch (damageMultiplier) {
                    case 1: return fire1;
                    case 2: return fire2;
                    case 3: return fire3;
                    case 4: return fire4;
                    case 5: return fire5;
                    default: return null;
                }
            case HEALTH: return health;
        }

        return null;
    }

    public Image getCharacter() {
        return character;
    }

}
