package evgeniy.martian.base.app;

import evgeniy.martian.base.objects.Manager;
import evgeniy.martian.base.objects.ResourceManager;

import javax.swing.*;
import java.awt.*;

public class UiApplication extends JFrame implements Application {

    public static final String TITLE = "Martian base";
    public static final Dimension MIN_SIZE = new Dimension(1280, 720);

    private final Manager manager;
    private final ResourceManager resourceManager;

    private PlayerInformation information = new PlayerInformation();
    private final LevelRenderer renderer;

    public UiApplication(Manager manager) throws HeadlessException {
        this.manager = manager;
        this.resourceManager = new ResourceManager(manager);
        this.renderer = new LevelRenderer(this, manager, resourceManager);
    }

    public void launch() {
        resourceManager.load();
        this.information.init();

        addKeyListener(renderer);

        setTitle(TITLE);
        setMinimumSize(MIN_SIZE);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension windowSize = getSize();
        setLocation(
                screenSize.width / 2 - windowSize.width / 2,
                screenSize.height / 2 - windowSize.height / 2
        );

        JPanel rootPanel = new JPanel(new BorderLayout());

//        rootPanel.add(this.information, BorderLayout.PAGE_START);
        rootPanel.add(this.renderer, BorderLayout.CENTER);

        getContentPane().add(rootPanel);

//        add(this.renderer);

        pack();
        setVisible(true);

        manager.setListener(new Manager.ActionListener() {
            @Override
            public void onUpdateStep(int steps) {
                UiApplication.this.information.updateSteps(steps);
            }

            @Override
            public void onUpdateHealth(int health) {
                UiApplication.this.information.updateHealth(health);
            }
        });
    }

    public LevelRenderer getRenderer() {
        return this.renderer;
    }

    @Override
    public void updateUi() {
        getRenderer().updateUI();
    }

    @Override
    public void completeDialog() {
        JOptionPane.showMessageDialog(this, "Рок успешно нашел выход", "Успешное завершение", JOptionPane.INFORMATION_MESSAGE);
    }

    @Override
    public void playerDeadDialog() {
        JOptionPane.showMessageDialog(this, "Рок умер", "Провальное завершение", JOptionPane.ERROR_MESSAGE);
    }

    @Override
    public void pathNotFoundDialog() {
        JOptionPane.showMessageDialog(this, "Рок не может найти выход", "Провальное завершение", JOptionPane.ERROR_MESSAGE);
    }

    @Override
    public void failureDialog() {
        JOptionPane.showMessageDialog(this, "В время выполнения произошла какая-то ошибка...", "Провальное завершение", JOptionPane.ERROR_MESSAGE);
    }

}
