package evgeniy.martian.base.objects;

import evgeniy.martian.base.level.tile.ButtonTile;
import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.Tile;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.IOException;

public class ResourceManager {

    private Image character;

    private Image floor;
    private Image wall;
    private Image exit;

    private Image cube;

    private Image doorA;
    private Image doorB;
    private Image doorC;
    private Image doorD;

    private Image buttonA;
    private Image buttonB;
    private Image buttonC;
    private Image buttonD;

    private Image bluePortal;
    private Image orangePortal;

    private final Manager manager;

    public ResourceManager(Manager manager) {
        this.manager = manager;
    }

    public void load() {
        character = load("/assets/character.jpg");

        floor = load("/assets/floor.jpg");
        wall = load("/assets/wall.jpg");
        exit = load("/assets/exit.jpg");

        cube = load("/assets/cube.jpg");

        doorA = load("/assets/door.jpg");
        doorB = load("/assets/door.jpg");
        doorC = load("/assets/door.jpg");
        doorD = load("/assets/door.jpg");
//
        buttonA = load("/assets/button.jpg");
        buttonB = load("/assets/button.jpg");
        buttonC = load("/assets/button.jpg");
        buttonD = load("/assets/button.jpg");

        orangePortal = load("/assets/orange-portal.jpg");
        bluePortal = load("/assets/blue-portal.jpg");
    }

    private Image load(String path) {
        try {
            return ImageIO.read(getClass().getResource(path));
        }
        catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Image getFor(Tile tile) {
        if (tile == null) return null;
        switch (tile.getType()) {
            case FLOOR: return floor;
            case WALL: return wall;
            case START: return floor;
            case EXIT: return exit;
            case DOOR:
                DoorTile.Variant doorVariant = ((DoorTile) tile).getVariant();

                boolean isPressed = manager.getLevel().buttonIsPressed(doorVariant.getButton());
                if (isPressed) {
                    return floor;
                }

                switch (doorVariant) {
                    case A: return doorA;
                    case B: return doorB;
                    case C: return doorC;
                    case D: return doorD;
                    default: return null;
                }
            case BUTTON:
                int x = tile.getX();
                int y = tile.getY();

                Tile cubeTile = manager.getLevel().getCube(x, y);
                if (cubeTile != null) {
                    return this.cube;
                }

                ButtonTile.Variant buttonVariant = ((ButtonTile) tile).getVariant();

                switch (buttonVariant) {
                    case A: return buttonA;
                    case B: return buttonB;
                    case C: return buttonC;
                    case D: return buttonD;
                    default: return null;
                }
            case ORANGE_PORTAL:
                return orangePortal;
            case BLUE_PORTAL:
                return bluePortal;
            case CUBE:
                return cube;
        }

        return null;
    }

    public Image getCharacter() {
        return character;
    }

}
