package evgeniy.martian.base.utils;

import evgeniy.martian.base.level.Level;
import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.Tile;
import evgeniy.martian.base.objects.Manager;

import java.util.ArrayDeque;
import java.util.Collection;
import java.util.Comparator;
import java.util.Deque;

import static evgeniy.martian.base.objects.pathfinding.AStarPathfinding.aStar;

public class ButtonsSolver {

    public static Deque<Position> solve(Manager manager, Position playerPosition, Position buttonPosition) {
        Level level = manager.getLevel();
        Deque<Deque<Position>> result = new ArrayDeque<>();
        cubeFor: for (Position cube : level.getCubes()) {
            Deque<Position> positions = aStar(manager, cube, buttonPosition);
            if (positions.isEmpty()) continue;


            for (Position position : positions) {
                Tile tile = level.get(position);
                if (tile.is(Tile.Type.DOOR)) {
                    DoorTile doorTile = (DoorTile) tile;
                    if (!level.buttonIsPressed(doorTile.getVariant().getButton())) {
                        continue cubeFor;
                    }
                }
            }

            result.add(positions);
        }

        return result.stream().sorted(Comparator.comparingInt(Collection::size)).findFirst().orElseThrow();
    }

}
