package evgeniy.martian.base.utils;

import evgeniy.martian.base.level.Level;
import evgeniy.martian.base.level.tile.CubeTile;
import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.Tile;
import evgeniy.martian.base.objects.Manager;

import java.util.ArrayList;
import java.util.Deque;
import java.util.List;

import static evgeniy.martian.base.objects.pathfinding.AStarPathfinding.aStar;

public class CubeUtils {

    public static List<Move> availableCubeMoves(Manager manager, Position cubePosition, Position playerPosition) {
        List<Move> result = new ArrayList<>();

        Position leftPos = cubePosition.plus(0, -1);
        if (canPass(manager, playerPosition, leftPos, cubePosition)) {
            result.add(Move.R);
        }

        Position rightPos = cubePosition.plus(0, 1);
        if (canPass(manager, playerPosition, rightPos, cubePosition)) {
            result.add(Move.L);
        }

        Position upPos = cubePosition.plus(-1, 0);
        if (canPass(manager, playerPosition, upPos, cubePosition)) {
            result.add(Move.D);
        }

        Position downPos = cubePosition.plus(1, 0);
        if (canPass(manager, playerPosition, downPos, cubePosition)) {
            result.add(Move.U);
        }

        return result;
    }

    private static boolean canPass(Manager manager, Position from, Position to, Position cube) {
        if (from.equals(to)) return true;

        Tile tile = manager.getLevel().get(to);
        if (!tile.canPass(manager)) return false;

        Deque<Position> positions = aStar(manager, from, to);
        for (Position position : positions) {
            if (position.equals(cube)) return false;
        }
        return true;
    }

    private static boolean canPassNow(Manager manager, Position from, Position to, Position cube) {
        Level level = manager.getLevel();
        if (!level.get(to).canPass(manager)) return false;

        Deque<Position> positions = aStar(manager, from, to);
        for (Position position : positions) {
            if (position == cube) return false;
        }
        return true;
    }

    private static boolean canPassNow(Manager manager, Position position, List<Position> ignores) {
        if (ignores.contains(position)) return true;

        Level level = manager.getLevel();
        Tile tile = level.get(position);
        if (!tile.canPassNow(manager)) {
            return false;
        }

        if (tile.is(Tile.Type.DOOR)) {
            boolean pressed = level.buttonIsPressed(((DoorTile) tile).getVariant().getButton());
            if (!pressed) {
                return false;
            }
        }

        CubeTile cube = level.getCube(position);
        return cube == null;
    }

    public enum Move {
        R(0, 1), L(0, -1), U(-1, 0), D(1, 0);

        private final int x;
        private final int y;

        Move(int x, int y) {
            this.x = x;
            this.y = y;
        }

        public Position plus(Position position) {
            return position.plus(x, y);
        }
        public Position sub(Position position) {
            return position.sub(x, y);
        }
    }

}
