package evgeniy.martian.base.utils;

import evgeniy.martian.base.level.Level;
import evgeniy.martian.base.level.tile.CubeTile;
import evgeniy.martian.base.level.tile.DoorTile;
import evgeniy.martian.base.level.tile.Tile;
import evgeniy.martian.base.objects.Manager;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;
import java.util.List;

import static evgeniy.martian.base.objects.pathfinding.AStarPathfinding.aStar;

public class GoalsFinder2 {

    private final Manager manager;
    private final Level level;

    public GoalsFinder2(Manager manager, Level level) {
        this.manager = manager;
        this.level = level;
    }

    public Deque<Position> findGoals() {

        Position currentPosition = level.getStartPosition();

//        Obstructions obstructions = findObstructions();
//        obstructions.dump();

//        Position currentPosition = manager.getPlayerPosition().toImmutable();
        List<Position> ignores = new ArrayList<>();

//        for (Position cube : obstructions.cubes) {
//            List<Move> moves = availableCubeMoves(cube, currentPosition, ignores);
//            ignores.add(cube);
//
//            if (moves.isEmpty()) throw new UnsupportedOperationException();
//
//            System.out.println(String.format("CUBE: %s - %s", cube, moves));
//        }

        return new ArrayDeque<>();
    }

    public Obstructions findObstructions(Position currPos) {
        Obstructions obstructions = new Obstructions();

        Level level = manager.getLevel();
        Position endPosition = level.getEndPosition();

        obstructions.path = aStar(manager, currPos, endPosition);
        for (Position position : obstructions.path) {
            Tile tile = level.get(position);
            if (tile.is(Tile.Type.DOOR)) {
                obstructions.doors.add(position);
            }

            CubeTile cube = level.getCube(position);
            if (cube != null) {
                obstructions.cubes.add(position);
            }
        }

        return obstructions;
    }

    private List<Move> availableCubeMoves(Position cubePosition, Position playerPosition, List<Position> ignores) {
        List<Move> result = new ArrayList<>();

        Position leftPos = cubePosition.plus(0, -1);
        if (canPassNow(playerPosition, leftPos, ignores)) {
            result.add(Move.R);
        }

        Position rightPos = cubePosition.plus(0, 1);
        if (canPassNow(playerPosition, rightPos, ignores)) {
            result.add(Move.L);
        }

        Position upPos = cubePosition.plus(-1, 0);
        if (canPassNow(playerPosition, upPos, ignores)) {
            result.add(Move.D);
        }

        Position downPos = cubePosition.plus(1, 0);
        if (canPassNow(playerPosition, downPos, ignores)) {
            result.add(Move.U);
        }

        return result;
    }

    private boolean canPassNow(Position from, Position to, List<Position> ignores) {
        if (!canPassNow(to, ignores)) return false;

        Deque<Position> positions = aStar(manager, from, to);
        for (Position position : positions) {
            if (!canPassNow(position, ignores)) return false;
        }
        return true;
    }

    private boolean canPassNow(Position position, List<Position> ignores) {
        if (ignores.contains(position)) return true;

        Tile tile = level.get(position);
        if (!tile.canPassNow(manager)) {
            return false;
        }

        if (tile.is(Tile.Type.DOOR)) {
            boolean pressed = level.buttonIsPressed(((DoorTile) tile).getVariant().getButton());
            if (!pressed) {
                return false;
            }
        }

        CubeTile cube = level.getCube(position);
        return cube == null;
    }

    public enum Move {

        R, L, U, D

    }

    public static class Obstructions {

        private List<Position> doors = new ArrayList<>();
        private List<Position> cubes = new ArrayList<>();

        private Deque<Position> path;

        public void dump() {
            System.out.println(String.format("Obstructions: doors: %s, cubes = %s", doors, cubes));
        }
    }

}
