package evgeniy.martian.base.utils;

import evgeniy.martian.base.level.Level;
import evgeniy.martian.base.objects.Manager;
import evgeniy.martian.base.objects.MoveTaskManager2;

import java.util.*;

import static evgeniy.martian.base.objects.pathfinding.AStarPathfinding.aStar;

public class PortalsConnector {

    public static Deque<MoveTaskManager2.Goal> connect(Manager manager, Position from, Position to) {
        Level level = manager.getLevel();
        List<Position> portals = level.getPortals();

        Deque<PortalState> states = new ArrayDeque<>();
        states.addFirst(new PortalState(null, from, to));
        while (!states.isEmpty()) {
            PortalState state = states.removeFirst();
            Deque<Position> path = aStar(manager, state.from, state.to);
            if (!path.isEmpty()) {
                return state.build();
            }

            Collection<PortalState> nextStates = state.calc(manager, portals);
            states.addAll(nextStates);
        }

        return new ArrayDeque<>();
    }

    private static class PortalState {

        PortalState parent;

        Position from;
        Position to;

        int len = 0;

        public PortalState(PortalState root, Position from, Position to) {
            this.parent = root;
            this.from = from;
            this.to = to;
        }

        public Collection<PortalState> calc(Manager manager, List<Position> portals) {
            Collection<PortalState> result = new ArrayList<>();

            for (Position portal : portals) {
                Deque<Position> path = aStar(manager, from, portal);
                if (path.isEmpty()) continue;

                if (len + 1 > portals.size()) continue;
                PortalState e = new PortalState(this, portal, to);
                e.len = len + 1;
                result.add(e);
            }

            return result;
        }

        public Deque<MoveTaskManager2.Goal> build() {
            Deque<MoveTaskManager2.Goal> result = new ArrayDeque<>();

            PortalState root = this.parent;
            while (root != null) {
                result.addLast(new MoveTaskManager2.Goal(root.from, true));
                root = root.parent;
            }

            return result;
        }

    }

}
