package evgeniy.martian.base.utils;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Path;
import java.util.Iterator;
import java.util.Objects;

public class Utils {

    public static String toDirectionSymbol(Position start, Position end) {
        if (start.equals(end)) return "";

        Position result = start.sub(end);
        int x = result.getX();
        int y = result.getY();

        if (y == -1) return "R";
        if (y == 1) return "L";
        if (x == -1) return "D";
        if (x == 1) return "U";

        throw new UnsupportedOperationException(x + " " + y + "; " + start + " " + end);
    }

    /**
     * @see java.nio.file.Files#write(Path, Iterable, Charset, OpenOption...)
     * changes: remove last new line
     */
    public static Path write(Path path, Iterable<? extends CharSequence> lines,
                             Charset cs, OpenOption... options)
            throws IOException
    {
        // ensure lines is not null before opening file
        Objects.requireNonNull(lines);
        CharsetEncoder encoder = cs.newEncoder();
        OutputStream out = Files.newOutputStream(path, options);
        try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(out, encoder))) {
            Iterator<? extends CharSequence> iterator = lines.iterator();
            if (iterator.hasNext()) {
                writer.append(iterator.next());
                while (iterator.hasNext()) {
                    writer.newLine();
                    writer.append(iterator.next());
                }
            }
        }
        return path;
    }

}
